import os
from django import template
from django.conf import settings

register = template.Library()

@register.simple_tag
def theme(file):
    theme_obj = settings.request_handler.theme
    media_path = os.path.join(theme_obj.media_root, file)
    theme_dev = getattr(settings, 'THEME_DEV', False)
    if not theme_dev:
        if os.path.exists(media_path):
            return os.path.join(theme_obj.media_url, file)
        else:
            static_url = getattr(settings, 'STATIC_URL', None)
            return os.path.join(static_url, file)
    else:
        return os.path.join('/themes', theme_obj.slug, 'media', file)


# -*- coding: utf-8 -*-
import shutil, os, errno

from optparse import make_option
from django.core.management.base import CommandError, BaseCommand
from django.utils.six.moves import input

from theming.core import ThemeManager

class Command(BaseCommand):

    theme_manager = ThemeManager()

    option_list = BaseCommand.option_list + (
        make_option('-l', '--link',
            action='store_true',
            dest='link',
            default=False,
            help='Create a symbolic link to each file instead of copying.'),
        make_option('-f', '--force',
            action='store_true',
            dest='force',
            default=False,
            help='Force to overwrite content.'),
    )

    def set_options(self, **options):
        self.link = options['link']
        self.force = options['force']

    def handle(self, *args, **options):
        self.set_options(**options)

        themes = self.theme_manager.search_themes()
        for key, theme in themes.iteritems():
            src = os.path.join(theme.theme_root, 'media')
            dest = theme.media_root
            self._copy_directory(src, dest, self.link, self.force)

    def _copy_directory(self, src, dest, use_symlinks=False, use_force=False):
        try:
            if use_force:
                shutil.rmtree(dest)
            shutil.copytree(src, dest, use_symlinks)
        except shutil.Error as e:
            print('Directory not copied. Error: %s' % e)
        except OSError as e:
            print('Directory not copied. Error: %s' % e)
            if e.errno == errno.EEXIST:
                message = ['\n']
                message.append('There are existing files in the destination.\n')
                message.append('Do you want to delete the destination\'s contents?.\n')
                message.append('Type y/n.\n')
                if input(''.join(message)) != 'y':
                    raise CommandError("Collecting themes files cancelled.")
                self.force = True
                self._copy_directory(src, dest, use_symlinks, True)


import re
from django.conf import settings
from theming.core import Theme, ThemeManager

class ThemingMiddleware(object):

    theme_manager = ThemeManager()

    def process_request(self, request):
        settings.request_handler = request
        request.theme = self.theme_manager.get_default()
        theme_config = getattr(settings, 'THEME_CONFIG', None)
        if theme_config:
            try:
                host = request.get_host()
                for (regex, value) in theme_config:
                    if re.search(regex, host):
                        request.theme = self.theme_manager.get_theme(value)
                        break
            except Theme.DoesNotExist:
                pass


import os
from django.conf import settings
from django.template.loaders.filesystem import Loader

class ThemeLoader(Loader):

    def get_template_sources(self, template_name, template_dirs=None):

        if not template_dirs:
            request = getattr(settings, 'request_handler', None)
            if request:
                template_dirs = (request.theme.template_dir,)

        return super(ThemeLoader, self).get_template_sources(
            template_name, template_dirs)


import os
from setuptools import setup

README = open(os.path.join(os.path.dirname(__file__), 'README.md')).read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name = 'django-theming',
    version = '0.1',
    packages = [
        'theming',
        'theming.management',
        'theming.management.commands',
        'theming.templatetags'
    ],
    include_package_data = True,
    description = 'Django app to implement the concept of theming, allow theming for host url.',
    long_description = README,
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)


import json, os
from django.conf import settings


class Theme(object):
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)
        
        theme_root = getattr(settings, 'THEME_ROOT', None)
        theme_media_root = getattr(settings, 'THEME_MEDIA_ROOT', None)
        theme_media_url = getattr(settings, 'THEME_MEDIA_URL', None)

        self.theme_root = os.path.join(theme_root, self.slug)
        self.media_root = os.path.join(theme_media_root, self.slug)
        self.media_url = os.path.join(theme_media_url, self.slug)
        
        self.template_dir = os.path.join(self.theme_root, 'templates')

class ThemeManager(object):

    _metadata = 'metadata.json'

    def __init__(self):
        self.theme_root = getattr(settings, 'THEME_ROOT', None)
        self.theme_default = getattr(settings, 'DEFAULT_THEME', 'default')
        self.themes = {}

    def add_theme(self, theme):
        if type(theme) is Theme:
            self.themes.update({theme.slug: theme})

    def get_theme(self, slug):
        try:
            if self.theme_root:
                metadata_path = os.path.join(
                    self.theme_root, slug, self._metadata
                )
                with open(metadata_path, 'r') as metadata:
                    theme = json.load(metadata)
                    return Theme(**theme)
            else:
                raise Exception('THEME_ROOT is not defined')
        except IOError as e:
            raise Exception(e)

    def get_default(self):
        return self.get_theme(self.theme_default)

    def search_themes(self):
        try:
            if self.theme_root:
                for (root, dirs, names) in os.walk(self.theme_root):
                    if self._metadata in names:
                        root_metadata = os.path.join(root, self._metadata)
                        with open(root_metadata, 'r') as metadata:
                            theme = json.load(metadata)
                            theme_obj = Theme(**theme)
                            self.add_theme(theme_obj)
                return self.themes
            else:
                raise Exception('THEME_ROOT is not defined')
        except IOError:
            raise Exception('Metadata is malformed')